@@ -1,5 +1,5 @@
 package Net::Async::AMQP::Channel;
-$Net::Async::AMQP::Channel::VERSION = '0.030';
+$Net::Async::AMQP::Channel::VERSION = '0.031';
 use strict;
 use warnings;
 
@@ -11,7 +11,7 @@ Net::Async::AMQP::Channel - represents a single channel in an MQ connection
 
 =head1 VERSION
 
-version 0.030
+version 0.031
 
 =head1 SYNOPSIS
 
@@ -469,9 +469,14 @@ sub on_close {
 	# ACK the close first - we have to send a close-ok
 	# before it's legal to reopen this channel ID
 	retain_future(
-		$self->send_frame(
-			Net::AMQP::Frame::Method->new(
-				method_frame => Net::AMQP::Protocol::Channel::CloseOk->new(
+		(
+			  # If we initiated the close, then the CloseOk comes from the server
+			  $self->{closing}
+			? Future->done
+			: $self->send_frame(
+				Net::AMQP::Frame::Method->new(
+					method_frame => Net::AMQP::Protocol::Channel::CloseOk->new(
+					)
 				)
 			)
 		)->then(sub {
@@ -525,6 +530,13 @@ sub close {
 	my %args = @_;
 	$self->debug_printf("Close channel %d", $self->id);
 
+	# There's a slight chance we'll get called after being
+	# removed from the loop, since we wanted to close anyway then
+	# don't treat that as an error
+	return Future->done if $self->{closing} or !$self->loop;
+
+	$self->{closing} = 1;
+
 	my $f = $self->loop->new_future;
 	my $frame = Net::AMQP::Frame::Method->new(
 		method_frame => Net::AMQP::Protocol::Channel::Close->new(
